#include "desktopfile.h"
#include "Files.h"
#include <qregexp.h>
#include <kdebug.h>
using namespace std;

DesktopFile::DesktopFile(QString const &fileName, bool readOnly, const char *resType):KDesktopFile(fileName, readOnly, resType)
{
}

QString DesktopFile::onlyShowIn() const
{
	return readEntry("OnlyShowIn");
}

QStringList DesktopFile::readCategories() const
{
	QString categories=readEntry("Categories");
	if(categories.isEmpty()) {
		// This is a MergeDir'ed desktop file -- take a good guess.
		QString file=fileName();
		// Ugly but necessary hardcodes for changed naming of commin
		// subtrees...
		categories=QString("Application;" + file.mid(categories.find("//")+2).replace(QRegExp("/"),";"));
		categories=categories
		          .replace(QRegExp("Internet"), "Network")
		          .replace(QRegExp("Games"), "Game")
		          .replace(QRegExp("Multimedia"), "AudioVideo")
		          .replace(QRegExp("Editors"), "Utility;Editor")
		          .replace(QRegExp("Preferences"), "Settings")
		          .replace(QRegExp("Toys"), "Game")
		          .replace(QRegExp("Utilities"), "Utility");
	}
	return QStringList::split(';', categories);
}

DesktopFiles::DesktopFiles():list<DesktopFile*>()
{
}

DesktopFiles::DesktopFiles(QString const &dir, bool subdirs, QStringList *sdirs):list<DesktopFile*>()
{
	addDirectory(dir, subdirs, sdirs);
}

DesktopFiles::~DesktopFiles()
{
	for(iterator it=begin(); it!=end(); it++)
		delete *it;
}

void DesktopFiles::addDirectory(QString const &dir, bool subdirs, QStringList *sdirs)
{
	if(dirs.contains(dir)) // No need to check a directory several times
		return;
	kdDebug(7122) << "Reading dir " << dir << "\n";
	dirs.append(dir);
	QStringList desktopFiles=Files::glob(dir + "/*.{desktop,kdelnk,directory}", (Files::Types)(Files::File | Files::Link));
	for(QStringList::Iterator it=desktopFiles.begin(); it!=desktopFiles.end(); it++) {
		kdDebug(7122) << "Adding " << *it << "\n";
		DesktopFile *d=new DesktopFile(*it);
#ifndef SHOW_EVERYTHING
		QString const nodisplay=d->readEntry("NoDisplay");
		QString const only=d->onlyShowIn();
		if(nodisplay != "true" && (only.isEmpty() || only.upper().contains("KDE")))
#endif
			insert(end(), d);
#ifndef SHOW_EVERYTHING
		else
			delete d;
#endif
	}
	if(subdirs) {
		QStringList dirs=Files::glob(dir + "/*", Files::Dir, false, false);
		for(QStringList::Iterator it=dirs.begin(); it!=dirs.end(); it++)
			if(*it != "." && *it != "..") {
				if(sdirs)
					sdirs->append(*it);
				addDirectory(*it, true, sdirs);
			}
	}
}

void DesktopFiles::addDirectories(QString const &dirs, bool subdirs, QStringList *sdirs)
{
	QStringList d=QStringList::split(':', dirs);
	for(QStringList::Iterator it=d.begin(); it!=d.end(); it++)
		addDirectory(*it, subdirs, sdirs);
}
