#include "kioentries.h"
#include "desktopfile.h"
#include <qfile.h>
#include <sys/stat.h>
#include <kdebug.h>
#include <klocale.h>
#include <kglobal.h>
using namespace KIO;

void KIOEntry::addAtom(unsigned int uds, long l)
{
	UDSAtom a;
	a.m_uds = uds;
	a.m_long = l;
	append(a);
}

void KIOEntry::addAtom(unsigned int uds, QString const &s)
{
	UDSAtom a;
	a.m_uds=uds;
	a.m_str = s;
	append(a);
}

DirEntry::DirEntry(QString const &name, QString const &desktop, QString const &baseurl, mode_t perms, size_t size):KIOEntry()
{
	if(!desktop.isEmpty()) {
		KConfig d(desktop, true);
		kdDebug(7122) << "KGlobal locale: " << KGlobal::locale()->language() << endl;
		d.setDesktopGroup();
		kdDebug(7122) << "Name for " << name << " in " << desktop << ": "  << d.readEntry("Name") << " Locale: " << d.locale() << endl;
		addAtom(KIO::UDS_NAME, d.readEntry("Name"));
		// Don't i18n the actual location
		if(!baseurl.isEmpty())
			addAtom(KIO::UDS_URL, baseurl + name + "/");
	} else {
		addAtom(KIO::UDS_NAME, QFile::decodeName(name.local8Bit()));
	}
	addAtom(KIO::UDS_FILE_TYPE, S_IFDIR);
	addAtom(KIO::UDS_ACCESS, perms);
	addAtom(KIO::UDS_SIZE, size);
#if 0
	if(!desktop.isEmpty()) {
		addAtom(KIO::UDS_URL, desktop);
	}
#endif
}

FileEntry::FileEntry(QString const &name, mode_t perms, size_t size, QString const &url, QString const &mimetype):KIOEntry()
{
	addAtom(KIO::UDS_NAME, QFile::decodeName(name.local8Bit()));
	addAtom(KIO::UDS_FILE_TYPE, S_IFREG);
	addAtom(KIO::UDS_ACCESS, perms);
	addAtom(KIO::UDS_SIZE, size);
	if(!url.isEmpty())
		addAtom(KIO::UDS_URL, url);
	if(!mimetype.isEmpty())
		addAtom(KIO::UDS_MIME_TYPE, mimetype);
}
