#include "parser.h"
#include <kdebug.h>
using namespace VFolder;
using namespace std;

bool XmlErrorHandler::error(const QXmlParseException &exception)
{
	kdError(7122) << "ERROR: XML parse error " << exception.message() << " at " << exception.publicId() << "/" << exception.systemId() << " in line " << exception.lineNumber() << ", column " << exception.columnNumber() << endl;
	return true;
}

bool XmlErrorHandler::fatalError(const QXmlParseException &exception)
{
	kdError(7122) << "FATAL ERROR: XML parse error " << exception.message() << " at " << exception.publicId() << "/" << exception.systemId() << " in line " << exception.lineNumber() << ", column " << exception.columnNumber() << endl;
	return true;
}

bool XmlErrorHandler::warning(const QXmlParseException &exception)
{
	kdWarning(7122) << "WARNING: XML parse error " << exception.message() << " at " << exception.publicId() << "/" << exception.systemId() << " in line " << exception.lineNumber() << ", column " << exception.columnNumber() << endl;
	return true;
}

QString XmlErrorHandler::errorString()
{
	return "Fatal error in XML parsing.";
}

VFolderParser::VFolderParser(DesktopFiles *d, VFolderEntry *r):QXmlDefaultHandler(),current(r),desktop(d),_not(false)
{
}

bool VFolderParser::startDocument()
{
	return true;
}

bool VFolderParser::startElement(QString const &nspace, QString const &localName, QString const &name, QXmlAttributes const &attr)
{
	content=QString::null;
	if(name=="Not") {
		_not=true;
		return true;
	}
	if(state==parsingQuery) {
		if(name=="And") {
			keywords.clear();
			state=parsingAndQuery;
		} else if(name=="Or") {
			keywords.clear();
			state=parsingOrQuery;
		}
	} else if(state==parsingAndQuery || state==parsingOrQuery) {
		if(name=="And" || name=="Or") {
			if(state==parsingAndQuery)
				current->addQuery(new VFolderQuery(keywords, VFolder::And, false));
			else
				current->addQuery(new VFolderQuery(keywords, VFolder::Or, false));
			keywords.clear();
			if(name=="And")
				state=parsingAndQuery;
			else
				state=parsingOrQuery;
		}
	} else {
		if(name=="Folder") {
			current=new VFolderEntry(Dir, current);
		} else if(name=="Query") {
			state=parsingQuery;
		}
	}
	return true;
}

bool VFolderParser::endElement(QString const &nspace, QString const &localName, QString const &name)
{
	if(name=="Not") {
		_not=false;
		return true;
	}
	if(state==parsingQuery) {
		if(name=="Query") {
			state=parsingFolder;
		}
	} else if(state==parsingAndQuery || state==parsingOrQuery) {
		if(name=="Keyword") {
			if(_not)
				keywords.append("!" + content);
			else
				keywords.append(content);
		} else if(state==parsingAndQuery && name=="And") {
			current->addQuery(new VFolderQuery(keywords, VFolder::And, false));
			state=parsingQuery;
		} else if(state==parsingOrQuery && name=="Or") {
			current->addQuery(new VFolderQuery(keywords, VFolder::Or, false));
			state=parsingQuery;
		}
	} else {
		if(name=="MergeDir") {
			QStringList subdirs;
			desktop->addDirectory(content, true, &subdirs);
		} else if(name=="DesktopDir") {
			desktop->addDirectory(content, false);
		} else if(name=="Name") {
			current->setName(content);
		} else if(name=="Desktop") {
			if(content.contains("/"))
				current->setDesktop(content);
			else {
				if(QFile::exists("/usr/share/desktop-menu-files/" + content))
					current->setDesktop("/usr/share/desktop-menu-files/" + content);
				else if(QFile::exists("/usr/share/gnome/vfolders/" + content))
					current->setDesktop("/usr/share/gnome/vfolders/" + content);
			}
		} else if(name=="OnlyUnallocated") {
		} else if(name=="DontShowIfEmpty") {
			current->setDontShowIfEmpty(true);
		} else if(name=="Folder") {
			current->parent()->append(current);
			current=current->parent();
		}
	}
	return true;
}

bool VFolderParser::characters(QString const &ch)
{
	content += ch;
	return true;
}
