#include "vfolder.h"
#include <kdebug.h>
using namespace VFolder;

VFolderEntry::VFolderEntry(Type t, VFolderEntry *parent):_parent(parent),_type(t),_dontshowifempty(false)
{
}

void VFolderEntry::addQuery(VFolderQuery *e)
{
	_query.append(e);
}

void VFolderEntry::addQueries(VFolderQueries *e)
{
	for(VFolderQueries::iterator it=e->begin(); it!=e->end(); it++)
		_query.append(*it);
}

VFolderQuery::VFolderQuery(QStringList const &query, QueryType type, bool reverse):_query(query),_type(type),_not(reverse)
{
}

bool VFolderQuery::match(DesktopFile const &d) const {
	bool found=(_type==And);
	QStringList categories=d.readCategories();
	if(categories.isEmpty())
		return false;
	kdDebug(7122) << "Parsing categories for " << d.readName() << endl;
	for(QStringList::ConstIterator it=_query.begin(); it!=_query.end(); it++) {
		if(_type==And) {
			if((*it).left(1)=="!")
				found = found && !categories.contains((*it).mid(1));
			else
				found = found && categories.contains(*it);
		} else if(_type==Or) {
			if((*it).left(1)=="!")
				found = found || !categories.contains((*it).mid(1));
			else
				found = found || categories.contains(*it);
		}
	}
	if(_not)
		found=!found;
#if 1
	if(found)
		kdDebug(7122) << "Matches " << (*this) << endl;
	else
		kdDebug(7122) << "Doesn't match " << (*this) << endl;
#endif
	return found;
}

// Useful for debugging...
VFolderQuery::operator QString() const {
	QString s;
	s = "[";
	if(_type==And)
		s += "AND-";
	else if(_type==Or)
		s += "OR-";
	s += "Query for " + _query.join(",") + "]";
	return s;
}

VFolderQueries::VFolderQueries():std::list<VFolderQuery*>() { }

VFolderQueries::~VFolderQueries() {
	for(iterator it=begin(); it!=end(); it++)
		delete *it;
}

bool VFolderQueries::match(DesktopFile const &d) const {
	const_iterator q=begin();
	if(q==end())
		return false;
	bool found=(*q)->match(d);
	while(++q!=end())
		found = found && (*q)->match(d);
	return found;
}
